@@ -61,7 +61,6 @@
 typedef struct
 {
   EmpathyIndividualManager *manager;
-  gboolean show_offline;
   gboolean show_avatars;
   gboolean show_groups;
   gboolean is_compact;
@@ -93,13 +92,13 @@ typedef struct
   EmpathyIndividualStore *self;
   FolksIndividual *individual;
   gboolean remove;
+  guint timeout;
 } ShowActiveData;
 
 enum
 {
   PROP_0,
   PROP_INDIVIDUAL_MANAGER,
-  PROP_SHOW_OFFLINE,
   PROP_SHOW_AVATARS,
   PROP_SHOW_PROTOCOLS,
   PROP_SHOW_GROUPS,
@@ -352,8 +351,7 @@ individual_store_add_individual (EmpathyIndividualStore *self,
 
   priv = GET_PRIV (self);
 
-  if (EMP_STR_EMPTY (folks_individual_get_alias (individual)) ||
-      (!priv->show_offline && !folks_individual_is_online (individual)))
+  if (EMP_STR_EMPTY (folks_individual_get_alias (individual)))
     return;
 
   if (priv->show_groups)
@@ -467,6 +465,26 @@ individual_store_contact_set_active (EmpathyIndividualStore *self,
 
 }
 
+static void individual_store_contact_active_free (ShowActiveData *data);
+
+static void
+individual_store_contact_active_invalidated (ShowActiveData *data,
+    GObject *old_object)
+{
+  /* Remove the timeout and free the struct, since the individual or individual
+   * store has disappeared. */
+  g_source_remove (data->timeout);
+
+  if (old_object == (GObject *) data->self)
+    data->self = NULL;
+  else if (old_object == (GObject *) data->individual)
+    data->individual = NULL;
+  else
+    g_assert_not_reached ();
+
+  individual_store_contact_active_free (data);
+}
+
 static ShowActiveData *
 individual_store_contact_active_new (EmpathyIndividualStore *self,
     FolksIndividual *individual,
@@ -482,13 +500,15 @@ individual_store_contact_active_new (EmpathyIndividualStore *self,
   /* We don't actually want to force either the IndividualStore or the
    * Individual to stay alive, since the user could quit Empathy or disable
    * the account before the contact_active timeout is fired. */
-  g_object_add_weak_pointer (G_OBJECT (self), (gpointer) &(data->self));
-  g_object_add_weak_pointer (G_OBJECT (individual),
-      (gpointer) &(data->individual));
+  g_object_weak_ref (G_OBJECT (self),
+      (GWeakNotify) individual_store_contact_active_invalidated, data);
+  g_object_weak_ref (G_OBJECT (individual),
+      (GWeakNotify) individual_store_contact_active_invalidated, data);
 
   data->self = self;
   data->individual = individual;
   data->remove = remove_;
+  data->timeout = 0;
 
   return data;
 }
@@ -496,27 +516,25 @@ individual_store_contact_active_new (EmpathyIndividualStore *self,
 static void
 individual_store_contact_active_free (ShowActiveData *data)
 {
+  if (data->self != NULL)
+    {
+      g_object_weak_unref (G_OBJECT (data->self),
+          (GWeakNotify) individual_store_contact_active_invalidated, data);
+    }
+
+  if (data->individual != NULL)
+    {
+      g_object_weak_unref (G_OBJECT (data->individual),
+          (GWeakNotify) individual_store_contact_active_invalidated, data);
+    }
+
   g_slice_free (ShowActiveData, data);
 }
 
 static gboolean
 individual_store_contact_active_cb (ShowActiveData *data)
 {
-  EmpathyIndividualStorePriv *priv;
-
-  /* They're weak pointers, so may have been NULLified between ShowActiveData
-   * being created and the timeout being fired. We assume they can only be
-   * destroyed in this thread, so this isn't MT-safe. */
-  if (data->self == NULL || data->individual == NULL)
-    {
-      individual_store_contact_active_free (data);
-      return FALSE;
-    }
-
-  priv = GET_PRIV (data->self);
-
-  if (data->remove &&
-      !priv->show_offline && !folks_individual_is_online (data->individual))
+  if (data->remove)
     {
       DEBUG ("Individual'%s' active timeout, removing item",
           folks_individual_get_alias (data->individual));
@@ -578,7 +596,6 @@ individual_store_contact_update (EmpathyIndividualStore *self,
   EmpathyContact *contact;
   GList *iters, *l;
   gboolean in_list;
-  gboolean should_be_in_list;
   gboolean was_online = TRUE;
   gboolean now_online = FALSE;
   gboolean set_model = FALSE;
@@ -606,46 +623,7 @@ individual_store_contact_update (EmpathyIndividualStore *self,
   /* Get online state now. */
   now_online = folks_individual_is_online (individual);
 
-  if (priv->show_offline || now_online)
-    {
-      should_be_in_list = TRUE;
-    }
-  else
-    {
-      should_be_in_list = FALSE;
-    }
-
-  if (!in_list && !should_be_in_list)
-    {
-      /* Nothing to do. */
-      DEBUG ("Individual:'%s' in list:NO, should be:NO",
-          folks_individual_get_alias (individual));
-
-      g_list_foreach (iters, (GFunc) gtk_tree_iter_free, NULL);
-      g_list_free (iters);
-      return;
-    }
-  else if (in_list && !should_be_in_list)
-    {
-      DEBUG ("Individual:'%s' in list:YES, should be:NO",
-          folks_individual_get_alias (individual));
-
-      if (priv->show_active)
-        {
-          do_remove = TRUE;
-          do_set_active = TRUE;
-          do_set_refresh = TRUE;
-
-          set_model = TRUE;
-          DEBUG ("Remove item (after timeout)");
-        }
-      else
-        {
-          DEBUG ("Remove item (now)!");
-          individual_store_remove_individual (self, individual);
-        }
-    }
-  else if (!in_list && should_be_in_list)
+  if (!in_list)
     {
       DEBUG ("Individual'%s' in list:NO, should be:YES",
           folks_individual_get_alias (individual));
@@ -746,7 +724,7 @@ individual_store_contact_update (EmpathyIndividualStore *self,
           data =
               individual_store_contact_active_new (self, individual,
               do_remove);
-          g_timeout_add_seconds (ACTIVE_USER_SHOW_TIME,
+          data->timeout = g_timeout_add_seconds (ACTIVE_USER_SHOW_TIME,
               (GSourceFunc) individual_store_contact_active_cb, data);
         }
     }
@@ -762,7 +740,7 @@ individual_store_contact_update (EmpathyIndividualStore *self,
 }
 
 static void
-individual_store_contact_updated_cb (FolksIndividual *individual,
+individual_store_individual_updated_cb (FolksIndividual *individual,
     GParamSpec *param,
     EmpathyIndividualStore *self)
 {
@@ -772,20 +750,45 @@ individual_store_contact_updated_cb (FolksIndividual *individual,
   individual_store_contact_update (self, individual);
 }
 
+static void
+individual_store_contact_updated_cb (EmpathyContact *contact,
+    GParamSpec *pspec,
+    EmpathyIndividualStore *self)
+{
+  FolksIndividual *individual;
+
+  DEBUG ("Contact '%s' updated, checking roster is in sync...",
+      empathy_contact_get_alias (contact));
+
+  individual = g_object_get_data (G_OBJECT (contact), "individual");
+  if (individual == NULL)
+    return;
+
+  individual_store_contact_update (self, individual);
+}
+
 static void
 individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
+  EmpathyContact *contact;
+
   g_signal_connect (individual, "notify::avatar",
-      G_CALLBACK (individual_store_contact_updated_cb), self);
+      G_CALLBACK (individual_store_individual_updated_cb), self);
   g_signal_connect (individual, "notify::presence-type",
-      G_CALLBACK (individual_store_contact_updated_cb), self);
+      G_CALLBACK (individual_store_individual_updated_cb), self);
   g_signal_connect (individual, "notify::presence-message",
-      G_CALLBACK (individual_store_contact_updated_cb), self);
+      G_CALLBACK (individual_store_individual_updated_cb), self);
   g_signal_connect (individual, "notify::alias",
+      G_CALLBACK (individual_store_individual_updated_cb), self);
+
+  /* FIXME: libfolks hasn't grown capabilities support yet, so we have to go
+   * through the EmpathyContact for them. */
+  contact = empathy_contact_dup_from_folks_individual (individual);
+  g_object_set_data (G_OBJECT (contact), "individual", individual);
+  g_signal_connect (contact, "notify::capabilities",
       G_CALLBACK (individual_store_contact_updated_cb), self);
-  g_signal_connect (individual, "notify::capabilities",
-      G_CALLBACK (individual_store_contact_updated_cb), self);
+  g_object_unref (contact);
 
   individual_store_add_individual (self, individual);
 }
@@ -795,6 +798,8 @@ individual_store_remove_individual_and_disconnect (
     EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
+  g_signal_handlers_disconnect_by_func (individual,
+      G_CALLBACK (individual_store_individual_updated_cb), self);
   g_signal_handlers_disconnect_by_func (individual,
       G_CALLBACK (individual_store_contact_updated_cb), self);
 
@@ -803,7 +808,7 @@ individual_store_remove_individual_and_disconnect (
 
 static void
 individual_store_members_changed_cb (EmpathyIndividualManager *manager,
-    gchar *message,
+    const gchar *message,
     GList *added,
     GList *removed,
     guint reason,
@@ -924,7 +929,7 @@ individual_store_member_renamed_cb (EmpathyIndividualManager *manager,
     FolksIndividual *old_individual,
     FolksIndividual *new_individual,
     guint reason,
-    gchar *message,
+    const gchar *message,
     EmpathyIndividualStore *self)
 {
   EmpathyIndividualStorePriv *priv;
@@ -955,6 +960,8 @@ individual_store_dispose (GObject *object)
   contacts = empathy_individual_manager_get_members (priv->manager);
   for (l = contacts; l; l = l->next)
     {
+      g_signal_handlers_disconnect_by_func (l->data,
+          G_CALLBACK (individual_store_individual_updated_cb), object);
       g_signal_handlers_disconnect_by_func (l->data,
           G_CALLBACK (individual_store_contact_updated_cb), object);
     }
@@ -999,9 +1006,6 @@ individual_store_get_property (GObject *object,
     case PROP_INDIVIDUAL_MANAGER:
       g_value_set_object (value, priv->manager);
       break;
-    case PROP_SHOW_OFFLINE:
-      g_value_set_boolean (value, priv->show_offline);
-      break;
     case PROP_SHOW_AVATARS:
       g_value_set_boolean (value, priv->show_avatars);
       break;
@@ -1039,10 +1043,6 @@ individual_store_set_property (GObject *object,
       individual_store_set_individual_manager (EMPATHY_INDIVIDUAL_STORE
           (object), g_value_get_object (value));
       break;
-    case PROP_SHOW_OFFLINE:
-      empathy_individual_store_set_show_offline (EMPATHY_INDIVIDUAL_STORE
-          (object), g_value_get_boolean (value));
-      break;
     case PROP_SHOW_AVATARS:
       empathy_individual_store_set_show_avatars (EMPATHY_INDIVIDUAL_STORE
           (object), g_value_get_boolean (value));
@@ -1085,12 +1085,6 @@ empathy_individual_store_class_init (EmpathyIndividualStoreClass *klass)
           "The individual manager",
           EMPATHY_TYPE_INDIVIDUAL_MANAGER,
           G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
-  g_object_class_install_property (object_class,
-      PROP_SHOW_OFFLINE,
-      g_param_spec_boolean ("show-offline",
-          "Show Offline",
-          "Whether contact list should display "
-          "offline contacts", FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
       PROP_SHOW_AVATARS,
       g_param_spec_boolean ("show-avatars",
@@ -1408,7 +1402,7 @@ individual_store_setup (EmpathyIndividualStore *self)
 }
 
 static gboolean
-individual_store_inibit_active_cb (EmpathyIndividualStore *self)
+individual_store_inhibit_active_cb (EmpathyIndividualStore *self)
 {
   EmpathyIndividualStorePriv *priv;
 
@@ -1432,7 +1426,7 @@ empathy_individual_store_init (EmpathyIndividualStore *self)
   priv->show_protocols = FALSE;
   priv->inhibit_active =
       g_timeout_add_seconds (ACTIVE_USER_WAIT_TO_ENABLE_TIME,
-      (GSourceFunc) individual_store_inibit_active_cb, self);
+      (GSourceFunc) individual_store_inhibit_active_cb, self);
   priv->status_icons =
       g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
   individual_store_setup (self);
@@ -1459,49 +1453,6 @@ empathy_individual_store_get_manager (EmpathyIndividualStore *self)
   return priv->manager;
 }
 
-gboolean
-empathy_individual_store_get_show_offline (EmpathyIndividualStore *self)
-{
-  EmpathyIndividualStorePriv *priv;
-
-  g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_STORE (self), FALSE);
-
-  priv = GET_PRIV (self);
-
-  return priv->show_offline;
-}
-
-void
-empathy_individual_store_set_show_offline (EmpathyIndividualStore *self,
-    gboolean show_offline)
-{
-  EmpathyIndividualStorePriv *priv;
-  GList *contacts, *l;
-  gboolean show_active;
-
-  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_STORE (self));
-
-  priv = GET_PRIV (self);
-
-  priv->show_offline = show_offline;
-  show_active = priv->show_active;
-
-  /* Disable temporarily. */
-  priv->show_active = FALSE;
-
-  contacts = empathy_individual_manager_get_members (priv->manager);
-  for (l = contacts; l; l = l->next)
-    {
-      individual_store_contact_update (self, l->data);
-    }
-  g_list_free (contacts);
-
-  /* Restore to original setting. */
-  priv->show_active = show_active;
-
-  g_object_notify (G_OBJECT (self), "show-offline");
-}
-
 gboolean
 empathy_individual_store_get_show_avatars (EmpathyIndividualStore *self)
 {
@@ -1729,6 +1680,9 @@ empathy_individual_store_set_sort_criterium (EmpathyIndividualStore *self,
       gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self),
           EMPATHY_INDIVIDUAL_STORE_COL_NAME, GTK_SORT_ASCENDING);
       break;
+
+    default:
+      g_assert_not_reached ();
     }
 
   g_object_notify (G_OBJECT (self), "sort-criterium");
