@@ -31,6 +31,7 @@
 
 #include <libxml/parser.h>
 #include <libxml/tree.h>
+#include <folks/folks.h>
 #include <telepathy-glib/account-manager.h>
 
 #include "empathy-contact.h"
@@ -74,7 +75,8 @@ guint        empathy_proxy_hash                     (gconstpointer    key);
 gboolean     empathy_check_available_state          (void);
 gint        empathy_uint_compare                    (gconstpointer a,
 						     gconstpointer b);
-const gchar * empathy_status_reason_get_default_message (TpConnectionStatusReason reason);
+
+const gchar * empathy_account_get_error_message (TpAccount *account);
 
 gchar *empathy_protocol_icon_name (const gchar *protocol);
 const gchar *empathy_protocol_name_to_display_name (const gchar *proto_name);
@@ -89,6 +91,11 @@ gboolean empathy_account_manager_get_accounts_connected (gboolean *connecting);
 void empathy_connect_new_account (TpAccount *account,
     TpAccountManager *account_manager);
 
+TpConnectionPresenceType empathy_folks_presence_type_to_tp (FolksPresenceType type);
+gboolean empathy_folks_individual_contains_contact (FolksIndividual *individual);
+EmpathyContact * empathy_contact_dup_from_folks_individual (FolksIndividual *individual);
+TpChannelGroupChangeReason tp_chanel_group_change_reason_from_folks_groups_change_reason (FolksGroupsChangeReason reason);
+
 G_END_DECLS
 
 #endif /*  __EMPATHY_UTILS_H__ */
