@@ -934,7 +934,7 @@ chat_window_invite_participant_activate_cb (GtkAction         *action,
 		const char *id;
 
 		id = empathy_contact_selector_dialog_get_selected (
-				EMPATHY_CONTACT_SELECTOR_DIALOG (dialog), NULL);
+				EMPATHY_CONTACT_SELECTOR_DIALOG (dialog), NULL, NULL);
 		if (EMP_STR_EMPTY (id)) goto out;
 
 		connection = tp_channel_borrow_connection (channel);
@@ -1738,14 +1738,8 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		}
 
 		if (!chat) {
-			TpConnection *connection;
-
-			connection = tp_account_get_connection (account);
-
-			if (connection) {
-				empathy_dispatcher_chat_with_contact_id (
-					connection, contact_id, gtk_get_current_event_time (), NULL, NULL);
-			}
+			empathy_dispatcher_chat_with_contact_id (
+				account, contact_id, gtk_get_current_event_time ());
 
 			g_strfreev (strv);
 			return;
@@ -1767,7 +1761,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		/* Added to take care of any outstanding chat events */
 		empathy_chat_window_present_chat (chat,
-			EMPATHY_DISPATCHER_NON_USER_ACTION);
+			TP_USER_ACTION_TIME_NOT_USER_ACTION);
 
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
@@ -2361,15 +2355,12 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
 	/* Don't force the window to show itself when it wasn't
 	 * an action by the user
 	 */
-	if (timestamp == EMPATHY_DISPATCHER_NON_USER_ACTION)
+	if (!tp_user_action_time_should_present (timestamp, &x_timestamp))
 		return;
 
 	priv = GET_PRIV (window);
 
-	if (timestamp == EMPATHY_DISPATCHER_CURRENT_TIME) {
-		x_timestamp = GDK_CURRENT_TIME;
-	} else {
-		x_timestamp = CLAMP (timestamp, 0, G_MAXUINT32);
+	if (x_timestamp != GDK_CURRENT_TIME) {
 		/* Don't present or switch tab if the action was earlier than the
 		 * last actions X time, accounting for overflow and the first ever
 		* presentation */
