@@ -29,7 +29,6 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/simple-approver.h>
 
-#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
@@ -76,7 +75,6 @@ typedef struct {
 } EventManagerApproval;
 
 typedef struct {
-  EmpathyDispatcher *dispatcher;
   TpBaseClient *approver;
   EmpathyContactManager *contact_manager;
   GSList *events;
@@ -310,14 +308,11 @@ event_manager_approval_approve (EventManagerApproval *approval)
 
   if (approval->auto_approved)
     {
-      timestamp = EMPATHY_DISPATCHER_NON_USER_ACTION;
+      timestamp = TP_USER_ACTION_TIME_NOT_USER_ACTION;
     }
   else
     {
-      timestamp = gtk_get_current_event_time ();
-
-      if (timestamp == GDK_CURRENT_TIME)
-        timestamp = EMPATHY_DISPATCHER_CURRENT_TIME;
+      timestamp = tp_user_action_time_from_x11 (gtk_get_current_event_time ());
     }
 
   g_assert (approval->operation != NULL);
@@ -336,9 +331,9 @@ static void
 event_text_channel_process_func (EventPriv *event)
 {
   EmpathyTpChat *tp_chat;
-  gint64 timestamp = gtk_get_current_event_time ();
-  if (timestamp == GDK_CURRENT_TIME)
-    timestamp = EMPATHY_DISPATCHER_CURRENT_TIME;
+  gint64 timestamp;
+
+  timestamp = tp_user_action_time_from_x11 (gtk_get_current_event_time ());
 
   if (event->approval->handler != 0)
     {
@@ -866,7 +861,7 @@ approve_channels (TpSimpleApprover *approver,
     {
       EmpathyTpChat *tp_chat;
 
-      tp_chat = empathy_tp_chat_new (channel);
+      tp_chat = empathy_tp_chat_new (account, channel);
       approval->handler_instance = G_OBJECT (tp_chat);
 
       if (tp_proxy_has_interface (channel, TP_IFACE_CHANNEL_INTERFACE_GROUP))
@@ -902,7 +897,7 @@ approve_channels (TpSimpleApprover *approver,
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
     {
       EmpathyContact *contact;
-      EmpathyTpCall *call = empathy_tp_call_new (channel);
+      EmpathyTpCall *call = empathy_tp_call_new (account, channel);
 
       approval->handler_instance = G_OBJECT (call);
 
@@ -924,7 +919,7 @@ approve_channels (TpSimpleApprover *approver,
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
       TpHandle handle;
-      EmpathyTpFile *tp_file = empathy_tp_file_new (channel, TRUE);
+      EmpathyTpFile *tp_file = empathy_tp_file_new (channel);
 
       approval->handler_instance = G_OBJECT (tp_file);
 
@@ -1118,7 +1113,6 @@ event_manager_finalize (GObject *object)
   g_slist_foreach (priv->approvals, (GFunc) event_manager_approval_free, NULL);
   g_slist_free (priv->approvals);
   g_object_unref (priv->contact_manager);
-  g_object_unref (priv->dispatcher);
   g_object_unref (priv->approver);
 }
 
@@ -1172,7 +1166,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
 
   manager->priv = priv;
 
-  priv->dispatcher = empathy_dispatcher_dup_singleton ();
   priv->contact_manager = empathy_contact_manager_dup_singleton ();
   g_signal_connect (priv->contact_manager, "pendings-changed",
     G_CALLBACK (event_manager_pendings_changed_cb), manager);
