@@ -91,6 +91,9 @@
 /* Name in the geometry file */
 #define GEOMETRY_NAME "main-window"
 
+/* Labels for empty contact list */
+#define NO_MATCH_FOUND _("No match found")
+
 G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
 
 #define GET_PRIV(self) ((EmpathyMainWindowPriv *)((EmpathyMainWindow *) self)->priv)
@@ -103,6 +106,7 @@ struct _EmpathyMainWindowPriv {
 	EmpathyEventManager     *event_manager;
 	guint                    flash_timeout_id;
 	gboolean                 flash_on;
+	gboolean                 empty;
 
 	GSettings              *gsettings_ui;
 	GSettings              *gsettings_contacts;
@@ -115,6 +119,8 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
 	GtkWidget              *search_bar;
+	GtkWidget              *notebook;
+	GtkWidget              *no_entry_label;
 
 	GtkToggleAction        *show_protocols;
 	GtkRadioAction         *sort_by_name;
@@ -356,6 +362,44 @@ OUT:
 	tp_clear_object (&individual);
 }
 
+static void
+main_window_row_deleted_cb (GtkTreeModel      *model,
+			    GtkTreePath       *path,
+			    EmpathyMainWindow *window)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkTreeIter help_iter;
+
+	if (!gtk_tree_model_get_iter_first (model, &help_iter)) {
+		priv->empty = TRUE;
+
+		if (empathy_individual_view_is_searching (
+				priv->individual_view)) {
+			gtk_label_set_text (GTK_LABEL (priv->no_entry_label),
+					NO_MATCH_FOUND);
+			gtk_notebook_set_current_page (
+					GTK_NOTEBOOK (priv->notebook),
+					0);
+		}
+	}
+}
+
+static void
+main_window_row_inserted_cb (GtkTreeModel      *model,
+			     GtkTreePath       *path,
+			     GtkTreeIter       *iter,
+			     EmpathyMainWindow *window)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+
+	if (priv->empty) {
+		priv->empty = FALSE;
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+				1);
+		gtk_widget_grab_focus (GTK_WIDGET (priv->individual_view));
+	}
+}
+
 static void
 main_window_remove_error (EmpathyMainWindow *window,
 			  TpAccount         *account)
@@ -417,8 +461,7 @@ main_window_error_close_clicked_cb (GtkButton         *button,
 
 static void
 main_window_error_display (EmpathyMainWindow *window,
-			   TpAccount         *account,
-			   const gchar       *message)
+			   TpAccount         *account)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	GtkWidget *info_bar;
@@ -435,7 +478,7 @@ main_window_error_display (EmpathyMainWindow *window,
 
 	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
 					       tp_account_get_display_name (account),
-					       message);
+					       empathy_account_get_error_message (account));
 
 	info_bar = g_hash_table_lookup (priv->errors, account);
 	if (info_bar) {
@@ -571,11 +614,7 @@ main_window_connection_changed_cb (TpAccount  *account,
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED &&
 	    reason != TP_CONNECTION_STATUS_REASON_REQUESTED) {
-		const gchar *message;
-
-		message = empathy_status_reason_get_default_message (reason);
-
-		main_window_error_display (window, account, message);
+		main_window_error_display (window, account);
 	}
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED) {
@@ -734,11 +773,8 @@ main_window_view_show_offline_cb (GtkToggleAction   *action,
 				EMPATHY_PREFS_UI_SHOW_OFFLINE,
 				current);
 
-	/* Turn off sound just while we alter the contact list. */
-	// FIXME: empathy_sound_set_enabled (FALSE);
-	empathy_individual_store_set_show_offline (priv->individual_store,
+	empathy_individual_view_set_show_offline (priv->individual_view,
 			current);
-	//empathy_sound_set_enabled (TRUE);
 }
 
 static void
@@ -902,7 +938,7 @@ static void
 main_window_view_show_map_cb (GtkCheckMenuItem  *item,
 			      EmpathyMainWindow *window)
 {
-#if HAVE_LIBCHAMPLAIN
+#ifdef HAVE_LIBCHAMPLAIN
 	empathy_map_view_show ();
 #endif
 }
@@ -912,16 +948,13 @@ join_chatroom (EmpathyChatroom *chatroom,
 	       gint64 timestamp)
 {
 	TpAccount      *account;
-	TpConnection   *connection;
 	const gchar    *room;
 
 	account = empathy_chatroom_get_account (chatroom);
-	connection = tp_account_get_connection (account);
-	g_assert (connection != NULL);
 	room = empathy_chatroom_get_room (chatroom);
 
 	DEBUG ("Requesting channel for '%s'", room);
-	empathy_dispatcher_join_muc (connection, room, timestamp);
+	empathy_dispatcher_join_muc (account, room, timestamp);
 }
 
 typedef struct
@@ -979,6 +1012,9 @@ account_status_changed_cb (TpAccount  *account,
 		case TP_CONNECTION_STATUS_CONNECTED:
 			/* We can join the room */
 			break;
+
+		default:
+			g_assert_not_reached ();
 	}
 
 	join_chatroom (ctx->chatroom, ctx->timestamp);
@@ -1491,6 +1527,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gboolean                  show_offline;
 	gchar                    *filename;
 	GSList                   *l;
+	GtkTreeModel             *model;
 
 	priv = window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window,
 			EMPATHY_TYPE_MAIN_WINDOW, EmpathyMainWindowPriv);
@@ -1519,6 +1556,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 				       "view_show_map", &show_map_widget,
 				       "room_join_favorites", &priv->room_join_favorites,
 				       "presence_toolbar", &priv->presence_toolbar,
+				       "notebook", &priv->notebook,
+				       "no_entry_label", &priv->no_entry_label,
 				       "roster_scrolledwindow", &sw,
 				       NULL);
 	g_free (filename);
@@ -1559,7 +1598,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	g_object_ref (priv->ui_manager);
 	g_object_unref (gui);
 
-#if !HAVE_LIBCHAMPLAIN
+#ifndef HAVE_LIBCHAMPLAIN
 	gtk_action_set_visible (show_map_widget, FALSE);
 #endif
 
@@ -1653,6 +1692,16 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	g_signal_connect_swapped (window, "map",
 		G_CALLBACK (gtk_widget_grab_focus), priv->individual_view);
 
+	/* Connect to proper signals to check if contact list is empty or not */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->individual_view));
+	priv->empty = TRUE;
+	g_signal_connect (model, "row-inserted",
+			  G_CALLBACK (main_window_row_inserted_cb),
+			  window);
+	g_signal_connect (model, "row-deleted",
+			  G_CALLBACK (main_window_row_deleted_cb),
+			  window);
+
 	/* Load user-defined accelerators. */
 	main_window_accels_load ();
 
